@@ -18,6 +18,8 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#include "config.h"
+
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -26,11 +28,9 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #endif
 #include <pwd.h>
 
-#include "config.h"
 #include "gnats.h"
 #include "pathmax.h"
 #include "globals.h"
-#include "pathmax.h"
 
 Responsible *responsible_chain;
 
@@ -57,12 +57,19 @@ find_submitter (cptr, site)
   if (err == -1) 
     return err;
 
+  memset (cptr, 0, sizeof (Submitter));
+
   if (err >= 5)
     cptr->notify = array[5];
   if (err >= 4)
     cptr->contact = array[4];
   if (err >= 3)
-    cptr->rtime = atoi (array[3]);
+    {
+      if (*array[3])
+	cptr->rtime = atoi (array[3]);
+      else
+	cptr->rtime = -1;	/* -1 => no response time specified */
+    }
   if (err >= 2)
     cptr->type = array[2];
   if (err >= 1)
@@ -73,8 +80,8 @@ find_submitter (cptr, site)
   if (cptr->contact == NULL)
     {
       log_msg (LOG_INFO, 1, "contact person not defined, setting to:",
-	       GNATS_ADMIN);
-      cptr->contact = GNATS_ADMIN;
+	       gnats_admin);
+      cptr->contact = gnats_admin;
     }
 
   return err;
@@ -137,6 +144,7 @@ find_responsible (person)
         return NULL;
 
       r = (Responsible *) xmalloc (sizeof (Responsible));
+      memset (r, 0, sizeof (Responsible));
 
       r->alias = array[2];
       r->fullname = array[1];
@@ -215,12 +223,15 @@ get_responsible_address (person)
   struct passwd *passwd;
   char *p;
 
-  /* First check the passwd file; if there's no entry, try the
-     responsible file.  */
-  passwd = getpwnam (person);
-  if (passwd == (struct passwd *)NULL)
-    responsible = find_responsible (person);
-  else
+  /* First check the responsible file; if there's no entry, try the
+     passwd file.  */
+  responsible = find_responsible (person);
+  if (responsible)
+    {
+      if (responsible->alias[0] == '\0')
+	responsible->alias = responsible->key;
+    }
+  else if ((passwd = getpwnam (person)) != 0)
     {
       responsible = (Responsible*) xmalloc (sizeof (Responsible));
       responsible->key = responsible->alias = (char *) strdup (person);
