@@ -113,6 +113,6 @@ extern char *xrealloc			PARAMS((char *, unsigned));
 extern void xfree			PARAMS((char *));
 
 /* in libiberty */
-extern char *basename           	PARAMS((char *));
+extern char *basename           	PARAMS((const char *));
 
 #endif /* !_globals_h  */
