@@ -18,6 +18,10 @@ You should have received a copy of the GNU General Public License
 along with GNU GNATS; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -33,13 +37,16 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "pathmax.h"
 
 static void version		PARAMS((void));
-static bool check_pr		PARAMS((FILE*));
-static void modify_pr		PARAMS((FILE*));
+static int check_pr		PARAMS((FILE*));
+static int modify_pr		PARAMS((FILE*));
 static int lock_pr		PARAMS((char*, char*));
-static void unlock_pr		PARAMS((char*));
+static int unlock_pr		PARAMS((char*));
 static char *get_path		PARAMS((char*));
 static void usage		PARAMS((void));
 
+extern char *basename		PARAMS((const char*));
+extern char *xmalloc		PARAMS((size_t));
+
 extern char *version_string;
 
 /* Name of this program.  */
@@ -52,7 +59,9 @@ int force = 0;
 enum {
   NONE,
   LOCK,
-  UNLOCK, 
+  UNLOCK,
+  LOCKDB,
+  UNLOCKDB,
   CHECK,
   MODIFY
 } edit_options;
@@ -61,6 +70,8 @@ struct option long_options[] =
 {
   {"lock", 1, NULL, 'l'},
   {"unlock", 0, NULL, 'u'},
+  {"lockdb", 0, NULL, 'L'},
+  {"unlockdb", 0, NULL, 'U'},
   {"check", 0, NULL, 'c'},
   {"directory", 1, NULL, 'd'},
   {"filename", 1, NULL, 'f'},
@@ -77,11 +88,11 @@ main (argc, argv)
 
   FILE *fp = stdin;
   int option;
-  char *fname;
+  char *fname = NULL;
   int result = 0;
   char *username = NULL;
 
-  program_name = (char *) basename (argv[0]);
+  program_name = basename (argv[0]);
 
   edit_options = MODIFY;
 
@@ -100,7 +111,7 @@ main (argc, argv)
 	     want to send mail to gnats-admin every time someone tries to
 	     edit a bogus PR number.  */
 	  if (fp == (FILE *)NULL)
-	    exit (1);
+	    exit (2);
 	  break;
 
 	case 'F':
@@ -116,18 +127,32 @@ main (argc, argv)
 	  edit_options = UNLOCK;
 	  break;
 
+	case 'L':
+	  edit_options = LOCKDB;
+	  break;
+
+	case 'U':
+	  edit_options = UNLOCKDB;
+	  break;
+
 	case 'c':
 	  edit_options = CHECK;
 	  break; 
 
 	case 'V':
 	  version ();
+	  exit (0);
 	  break;
 	  
 	case 'h':
-	  /* Fall through.  */
+	  usage ();
+	  exit (0);
+	  break;
+	  
 	default:
 	  usage ();
+	  exit (3);
+	  break;
 	}
     }
 
@@ -135,33 +160,45 @@ main (argc, argv)
   if (optind < argc)
     fname = argv[optind];
  
+  configure ();
   init_gnats ();
+  umask (022);
+  
+  block_signals ();
 
-  lock_gnats ();
+  if (edit_options != UNLOCKDB)
+    lock_gnats ();
   
   switch (edit_options) {
     case LOCK:
       result = lock_pr (fname, username);
       break;
     case UNLOCK:
-      unlock_pr (fname);
+      result = unlock_pr (fname);
       break;
     case CHECK:
       result = check_pr (fp);
       break;
+    case LOCKDB:
+    case UNLOCKDB:
+      result = 1; /* so that the program exits with 0 status */
+      break;
     default: 
-      modify_pr (fp);
+      result = modify_pr (fp);
       break;
   }
 
   fclose (fp);
 
-  unlock_gnats ();
+  if (edit_options != LOCKDB)
+    unlock_gnats ();
 
-  exit (result);
+  unblock_signals ();
+
+  exit (!result);
 }
 
-static void
+static int
 modify_pr (fp)
     FILE *fp;
 {
@@ -180,14 +217,14 @@ modify_pr (fp)
 
   /* this will print the bad values out. then return and exit.  */
   if (!check_pr (fp))
-    return;
+    return 0;
 
   /* build the new index entry.  */
   new_index->category = field_value (CATEGORY);
   new_index->number = field_value (NUMBER);
   new_index->submitter = field_value (SUBMITTER);
   new_index->responsible = field_value (RESPONSIBLE);
-  if (new_index->responsible)
+  if (*new_index->responsible)
     {
       p = (char *) strchr (new_index->responsible, ' ');
       if (p != NULL)
@@ -270,8 +307,6 @@ modify_pr (fp)
     sprintf (path, "%s/%s/%s", gnats_root, new_index->category,
 	     new_index->number);
 
-  block_signals ();
-
   if (! force && rename (old_path, lock_path) < 0)
     {
       if (errno != EXDEV)
@@ -314,13 +349,13 @@ modify_pr (fp)
       write_index (index_chain);
     }
 
-  unblock_signals ();
+  return 1;
 }
 
 /* check_pr - reads the PR in, searches for bogus enum types, lack of
    a PR number, and prints out the bogus info.  */
 
-static bool
+static int
 check_pr (fp)
      FILE* fp;
 {
@@ -332,23 +367,24 @@ check_pr (fp)
   read_pr (fp, 0);
 
   enums = check_enum_types (2);
-  if (enums != (struct bad_enum *) NULL)
-    {
-      for (e = enums; e; e = e->next)
-        printf ("%s", e->msg);
-
-      return FALSE;
-    }
-
-  /* It was missing its number! */
   p = field_value (NUMBER);
-  if (p != NULL && *p == '-')
+
+  if (enums || *p == '-')
     {
-      printf ("Number: %s\n", p);
-      return FALSE;
+      printf ("pr-edit: invalid fields");
+      
+      if (enums)
+	for (e = enums; e; e = e->next)
+	  printf ("%s", e->msg);
+      
+      /* It was missing its number! */
+      if (*p == '-')
+	printf ("Number: %s\n", p);
+      
+      return 0;
     }
 
-  return TRUE;
+  return 1;
 }
 
 static int
@@ -360,7 +396,7 @@ lock_pr (fname, user)
   FILE *ifp;
 
   if (fname == NULL)
-    return 1;
+    return 0;
 
   lock_path = get_path (fname);
 
@@ -372,17 +408,15 @@ lock_pr (fname, user)
       /* If we couldn't open it for reading, something else is hosed,
 	 so just bail now.  */
       if (fp == (FILE *) NULL)
-	return 1;
+	return 0;
 
       fscanf (fp, "%s", buf);
       fprintf (stderr, "%s: PR %s locked by %s\n", program_name,
 	       fname, buf);
       fclose (fp);
-      return 1;
+      return 0;
     }
 
-  block_signals ();
-
   ifp = fopen (lock_path, "w");
   if (ifp == NULL)
     punt (1, "%s: can't create lock file %s", program_name, lock_path);
@@ -390,12 +424,10 @@ lock_pr (fname, user)
   fprintf (ifp, "%s\n", user);
   fclose (ifp);
 
-  unblock_signals ();
-
-  return 0;
+  return 1;
 }
 
-static void
+static int
 unlock_pr (fname)
     char *fname;
 {
@@ -403,29 +435,40 @@ unlock_pr (fname)
   struct stat buf;
 
   if (fname == NULL)
-    return;
+    return 0;
 
   lock_path = get_path (fname);
 
-  if (stat (lock_path, &buf) != 0)
-    punt (1, "%s: lock file %s does not exist", program_name, lock_path);
+  if (lock_path == NULL)
+    {
+      fprintf (stderr, "%s: no such PR `%s'\n", program_name, fname);
+      return 0;
+    }
 
-  block_signals ();
+  if (stat (lock_path, &buf) != 0)
+    {
+      fprintf (stderr, "%s: lock file %s does not exist\n",
+	       program_name, lock_path);
+      return 0;
+    }
 
   if (unlink (lock_path) == -1)
-    punt (1, "%s: can't unlink lock file %s ", program_name, lock_path);
+    {
+      fprintf (stderr, "%s: can't delete lock file %s\n",
+	       program_name, lock_path);
+      return 0;
+    }
 
-  unblock_signals ();
+  return 1;
 }
 
 static void
 usage ()
 {
   fprintf (stderr, "\
-Usage: %s [-cuV] [-d directory] [-f check-filename] [-l username]\n\
-[--directory=directory] [--filename=check-filename] [--lock=user]\n\
-[--unlock] [--help] [--version] [--check] [PR]\n", program_name);
-  exit (1);
+Usage: %s [-cuVFh] [-d directory] [-f filename] [-l username]\n\
+          [--directory=directory] [--filename=filename] [--lock=user]\n\
+          [--unlock] [--help] [--version] [--check] [PR]\n", program_name);
 }
 
 static void
@@ -442,7 +485,7 @@ get_path (fname)
   char *p;
   FILE *ifp;
   char *category = NULL; 
-  char *path = (char *) xmalloc (PATH_MAX);
+  char *path;
 
   if (fname == NULL)
     return NULL;
@@ -461,6 +504,7 @@ get_path (fname)
         return NULL;
     }
 
+  path = (char *) xmalloc (PATH_MAX);
   if (category)
     sprintf (path, "%s/%s/%s.lock", gnats_root, category, fname);
   else
